[run]
branch = True
cover_pylib = False
relative_files = True

omit =
  .nox/*
  setup.py

[report]
# Regexes for lines to exclude from consideration
exclude_lines =
    # Have to re-enable the standard pragma
    pragma: no cover

    # Don't complain about missing debug-only code:
    def __repr__

    # Don't complain if tests don't hit defensive assertion code:
    raise AssertionError
    raise NotImplementedError

    # Don't complain if non-runnable code isn't run:
    if 0:
    if False:
    if __name__ == .__main__.:

omit =
  .nox/*
  setup.py


ignore_errors = True

[paths]
source =
  saltext/{{ project_name }}
  src/saltext/{{ project_name }}
testsuite =
  tests/
